(*
    A simple metronome - Thread for counting clock ticks
    Copyright (c) 1998-2003 Alessandro Scotti
    www.ascotti.org

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
unit Clock;

interface

uses Classes;

type
  TClock = class(TThread)
  private
    FDelay: integer;
    FTicks: integer;
    FOnNotify: TNotifyEvent;
    procedure DoNotify;
    procedure SetDelay( Value: integer );
    procedure SetTicks( Value: integer );
  public
    constructor Create( CreateSuspended: boolean );
    procedure Execute; override;
    property Ticks: integer read FTicks write SetTicks;
    property Delay: integer read FDelay write SetDelay;
    property OnNotify: TNotifyEvent read FOnNotify write FOnNotify;
  end;

implementation

uses Windows;

constructor TClock.Create( CreateSuspended: boolean );
begin
  inherited Create( true );
  FOnNotify := nil;
  Ticks := 60;
  if( not CreateSuspended ) then
    Resume;
end;

procedure TClock.DoNotify;
begin
  FOnNotify( Self );
end;

procedure TClock.Execute;
var
  STime: integer;
  ETime: integer;
begin
  STime := GetTickCount;
  while( not Terminated ) do begin
    Sleep( (Delay * 3) div 4 );
    repeat
      ETime := GetTickCount;
    until( ETime-STime > Delay );
    STime := ETime;
    if( Assigned(FOnNotify) ) then
      Synchronize( DoNotify );
  end;
end;

procedure TClock.SetTicks( Value: integer );
begin
  if( Value <> FTicks ) then begin
    FTicks := Value;
    // Ticks are beats per minute
    Delay := 60000 div FTicks;
  end;
end;

procedure TClock.SetDelay( Value: integer );
begin
  if( Value <> FDelay ) then begin
    Suspend;
    FDelay := Value;
    Resume;
  end;
end;

end.
